@@ -6,11 +6,14 @@ import io
 import json
 import os
 import sys
+import uuid
 from asyncio import AbstractEventLoop
 from datetime import datetime
 from decimal import Decimal
+from pathlib import Path
 from tempfile import mkstemp
 from threading import Thread
+from typing import Dict
 
 import pkg_resources
 from c8connector import C8Connector, Sample, ConfigAttributeType, Schema
@@ -102,6 +105,26 @@ class PostgresTargetConnector(C8Connector):
                                        'rows requires metadata columns, `hard_delete` option automatically enables the'
                                        ' `add_metadata_columns` option as well.',
                            default_value='false'),
+            ConfigProperty('ssl', 'Use SSL', ConfigAttributeType.BOOLEAN, False, False,
+                           description='If set to `true` then use SSL via postgres sslmode `require` option.'
+			   	                       'If the server does not accept SSL connections or the client certificate is not recognized'
+			                           ' then the connection will fail.',
+                           default_value='false'),
+            ConfigProperty('ssl_root_ca_cert', 'SSL CA Certificate', ConfigAttributeType.STRING, False, False,
+                           description='Specific CA certificate in PEM string format. This is most often the case '
+                                       'when using `self-signed` server certificate.',
+                           placeholder_value="my_ca_certificate"),
+            ConfigProperty('ssl_client_certificate', 'SSL Client Certificate', ConfigAttributeType.STRING, False, False,
+                           description='Specific client certificate in PEM string format. The private key for client '
+                                       'certificate should be specfied in a different parameter, SSL Client Key.',
+                           placeholder_value="my_client_certificate"),
+            ConfigProperty('ssl_client_key', 'SSL Client Key', ConfigAttributeType.STRING, False, False,
+                           description='Specific client key in PEM string format.',
+                           placeholder_value="my_client_key"),
+            ConfigProperty('ssl_client_password', 'SSL Client Password', ConfigAttributeType.STRING, False, False,
+                           description='If the private key contained in the SSL Client Key is encrypted, users can provide a '
+                                       'password or passphrase to decrypt the encrypted private keys.',
+                           placeholder_value="my_client_password"),
             ConfigProperty('batch_size_rows', 'Batch Size', ConfigAttributeType.INT, False, False,
                            description='Maximum number of rows inserted per batch.',
                            default_value='100000'),
@@ -539,6 +562,83 @@ def start_background_loop(loop: asyncio.AbstractEventLoop) -> None:
     asyncio.set_event_loop(loop)
     loop.run_forever()
 
+def create_certficate_files(config: Dict) -> Dict:
+    path_uuid = uuid.uuid4().hex
+    try:
+        if config.get('ssl_root_ca_cert'):
+            path = f"/opt/postgresql/{path_uuid}/ca.crt"
+            ca_cert = Path(path)
+            ca_cert.parent.mkdir(exist_ok=True, parents=True)
+            ca_cert.write_text(create_ssl_string(config['ssl_root_ca_cert']))
+	    ca_cert.chmod(0o600)
+            config['ssl_root_ca_cert'] = path
+            LOGGER.info(f"CA certificate file created at: {path}")
+
+        if config.get('ssl_client_certificate'):
+            path = f"/opt/postgresql/{path_uuid}/client.crt"
+            client_cert = Path(path)
+            client_cert.parent.mkdir(exist_ok=True, parents=True)
+            client_cert.write_text(create_ssl_string(config['ssl_client_certificate']))
+	    client_cert.chmod(0o600)
+            config['ssl_client_certificate'] = path
+            LOGGER.info(f"Client certificate file created at: {path}")
+
+        if config.get('ssl_client_key'):
+            path = f"/opt/postgresql/{path_uuid}/client.key"
+            client_cert = Path(path)
+            client_cert.parent.mkdir(exist_ok=True, parents=True)
+            client_cert.write_text(create_ssl_string(config['ssl_client_key']))
+	    client_cert.chmod(0o600)
+            config['ssl_client_key'] = path
+            LOGGER.info(f"Client key file created at: {path}")
+    except Exception as e:
+        LOGGER.warn(f"Failed to create certificate: /opt/postgresql/{path_uuid}/. {e}")
+    return config
+
+def delete_certficate_files(config: Dict) -> None:
+    try:
+        cert = None
+        if config.get('ssl_root_ca_cert'):
+            path = config['ssl_root_ca_cert']
+            cert = Path(path)
+            config['ssl_root_ca_cert'] = cert.read_text()
+            cert.unlink()
+            LOGGER.info(f"CA certificate file deleted from: {path}")
+
+        if config.get('ssl_client_certificate'):
+            path = config['ssl_client_certificate']
+            cert = Path(path)
+            config['ssl_client_certificate'] = cert.read_text()
+            cert.unlink()
+            LOGGER.info(f"Client certificate file deleted from: {path}")
+
+        if config.get('ssl_client_key'):
+            path = config['ssl_client_key']
+            cert = Path(path)
+            config['ssl_client_key'] = cert.read_text()
+            cert.unlink()
+            LOGGER.info(f"Client key file deleted from: {path}")
+
+        if cert is not None:
+            cert.parent.rmdir()
+    except Exception as e:
+        LOGGER.warn(f"Failed to delete certificate: {e}")
+
+def create_ssl_string(ssl_string: str) -> str:
+    tls_certificate_key_list = []
+    split_string = ssl_string.split("-----")
+    if len(split_string) < 4:
+        raise Exception("Invalid PEM format for certificate.")
+    for i in range(len(split_string)):
+        if((i % 2) == 1):
+            tls_certificate_key_list.append("-----")
+            tls_certificate_key_list.append(split_string[i])
+            tls_certificate_key_list.append("-----")
+        else:
+            tls_certificate_key_list.append(split_string[i].replace(' ', '\n'))
+
+    tls_certificate_key_file = ''.join(tls_certificate_key_list)
+    return tls_certificate_key_file
 
 def main():
     """Main entry point"""
@@ -552,11 +652,17 @@ def main():
     else:
         config = {}
 
-    # Consume singer messages
-    singer_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
-    persist_lines(config, singer_messages)
+    try:
+        config = create_certficate_files(config)
+        # Consume singer messages
+        singer_messages = io.TextIOWrapper(sys.stdin.buffer, encoding='utf-8')
+        persist_lines(config, singer_messages)
 
-    LOGGER.debug("Exiting normally")
+        LOGGER.debug("Exiting normally")
+    except Exception as e:
+        delete_certficate_files(config)
+        raise e
+    delete_certficate_files(config)
 
 
 if __name__ == '__main__':
