@@ -304,8 +304,16 @@ class DbSync:
             self.connection_config['port']
         )
 
-        if 'ssl' in self.connection_config and self.connection_config['ssl'] == 'true':
+        if self.connection_config.get('ssl', False):
             conn_string += " sslmode='require'"
+            if self.connection_config.get('ssl_root_ca_cert'):
+                conn_string += " sslrootcert='{}'".format(self.connection_config['ssl_root_ca_cert'])
+            if self.connection_config.get('ssl_client_certificate'):
+                conn_string += " sslcert='{}'".format(self.connection_config['ssl_client_certificate'])
+                if self.connection_config.get('ssl_client_key'):
+                    conn_string += " sslkey='{}'".format(self.connection_config['ssl_client_key'])
+                if self.connection_config.get('ssl_client_password'):
+                    conn_string += " sslpassword='{}'".format(self.connection_config['ssl_client_password'])
 
         return psycopg2.connect(conn_string)
 
